==> arepoicgen-0.1.18-py3-none-any.whl: arepoICgen/densityPerturbations.py <==
# Importing libraries
import numpy as np

# Some useful constants
G = 6.67e-8
kB = 1.38e-16

# Function to apply a BB density fluctuation
def bossBodenheimer(ngas, pos, mass):
    # Calculate the centre of mass
    totMass = np.sum(mass)
    xCom = np.sum(pos[0] * mass) / totMass
    yCom = np.sum(pos[1] * mass) / totMass

    # Apply the density perturbation
    for i in range(ngas):
        # Find relative positions
        x = xCom - pos[0,i]
        y = yCom - pos[1,i]

        # Work out the angle 
        phi = np.arctan2(y, x)

        # Work out what the mass should be here
        mass[i] = mass[i] * (1 + 0.5 * np.cos(2*phi))

    return pos, mass

def densityGradient(pos, mass, lowerDensity=0.66, upperDensity=1.33):
    distance = pos[0] + np.max(pos[0])
    return lowerDensity * mass + (upperDensity-lowerDensity) * mass * (distance/np.max(distance))

def bonnorEbert(ngas, pos, mass, temp, mu, beMass):
    # Calculate the sound speed
    cs = np.sqrt(kB * temp / (mu * 1.66e-24))
    beMass = beMass * 1.991e33
    
    # Calculate characteristic quantities 
    rBonnorEbert = 3.09e18 * 0.0043016 * (beMass/1.991e33) * 6.5 / ((cs/1e5)**2 * 15.85)
    print("Bonnor Ebert Radius: {:.2e}".format(rBonnorEbert))
     
    outsideRho = 1.66e-22
    centralDensity = 14.305 * outsideRho
    rCharacteristic = cs / (np.sqrt(4 * np.pi * G * centralDensity))

    #centralDensity = (6.5**2 / np.sqrt(4 * np.pi)) * (cs**2 / G) * (1/rBonnorEbert**2)
    print("Central Density: {:.2e}".format(centralDensity))
    #rCharacteristic = cs / np.sqrt(4 * np.pi * G * centralDensity)
    print("Characteristic Radius: {:.2e}".format(rCharacteristic))
    
    # Create bins of radius and corresponding density
    radiusBins = 10**np.linspace(0, np.log10(rBonnorEbert), 1000)
    densityBins = centralDensity * rCharacteristic**2 / (rCharacteristic**2 + radiusBins**2)

    # Find the centre of mass
    totMass = np.sum(mass)
    xC = np.sum(pos[0] * mass) / totMass
    yC = np.sum(pos[1] * mass) / totMass
    zC = np.sum(pos[2] * mass) / totMass
    
    # Find each particle's distance to the COM
    rCentre = np.sqrt((pos[0])**2 + (pos[1])**2 + (pos[2])**2)
    
    # Scale everything to the correct radius
    pos[0] = rBonnorEbert * pos[0] / np.max(rCentre) 
    pos[1] = rBonnorEbert * pos[1] / np.max(rCentre)
    pos[2] = rBonnorEbert * pos[2] / np.max(rCentre)
    rCentre = rBonnorEbert * rCentre / np.max(rCentre) 
    
    # Scale the mass of each particle
    for i in range(len(radiusBins)-1): 
        inRadius = np.where((rCentre > radiusBins[i]) & (rCentre < radiusBins[i+1]))
            
        # Work out how much mass should be in each shell
        shellMass = (4 * np.pi / 3) * (radiusBins[i+1]**3 - radiusBins[i]**3) * densityBins[i] 
        
        # Find the total mass of particles in this shell and scale
        cellMass = shellMass / len(inRadius[0])
        
        # Assign value
        mass[inRadius] = cellMass
        
    # Scale masses
    totMass = np.sum(mass)
    mass = mass * (beMass / totMass)
        
    # Work out cloud density
    volume = ((4 * np.pi / 3) * rBonnorEbert**3)
    cloudDensity = beMass / volume
    densityFrac = np.min(densityBins)/cloudDensity

    return mass, pos, densityFrac/15, volume/(3.09e18**3)

# Create a simple centrally-condensed density profile
def centrallyCondensedSphere(ngas, pos, pMass, mass, rFlat, densityGradient=-1.5):
    # Calculate the centre of mass
    xcom = np.sum(pos[0] * pMass) / np.sum(pMass)
    ycom = np.sum(pos[1] * pMass) / np.sum(pMass)
    zcom = np.sum(pos[2] * pMass) / np.sum(pMass)
    
    # Find radial distance to the CoM
    rCentre = np.sqrt((pos[0] - xcom)**2 + (pos[1] - ycom)**2 + (pos[2] - zcom)**2)
    
    # Scale the masses 
    densityProfile = (rFlat**(-1 * densityGradient + 1) / (rFlat**(-1 * densityGradient + 1) + rCentre**(-1 * densityGradient + 1)) / np.max(rCentre))
    
    rBins = np.linspace(0, np.max(rCentre), 1000)
    for i in range(len(rBins)-1):
        inBin = np.where((rCentre > rBins[i]) & (rCentre < rBins[i+1]))
        
        vol = 4 * np.pi * (rBins[i+1]**3 - rBins[i]**3) / 2
        massInBin = vol * (rFlat**(-1 * densityGradient + 1) / (rFlat**(-1 * densityGradient + 1) + ((rBins[i]+rBins[i+1])/2)**(-1 * densityGradient + 1)) / np.max(rCentre))
        
        cellMass = massInBin / len(inBin[0])
        pMass[inBin] = cellMass
        
    #pMass = pMass[0] * (rFlat**(-1 * densityGradient + 1) / (rFlat**(-1 * densityGradient + 1) + rCentre**(-1 * densityGradient + 1)) / np.max(rCentre))
    
    #pMass = pMass[0] * rCentre**(densityGradient - 1)
    #pMass = pMass[0] * rFlat**2 / (rFlat**2 + rCentre**2)
    pMass = pMass * (mass*1.991e33 / np.sum(pMass))
    
    densityFraction = 0.1 * np.min(pMass) / np.mean(pMass)
        
    return pos, pMass, densityFraction

==> arepoicgen-0.1.19-py3-none-any.whl: arepoICgen/densityPerturbations.py <==
# Importing libraries
import numpy as np

# Some useful constants
G = 6.67e-8
kB = 1.38e-16

# Function to apply a BB density fluctuation
def bossBodenheimer(ngas, pos, mass):
    # Calculate the centre of mass
    totMass = np.sum(mass)
    xCom = np.sum(pos[0] * mass) / totMass
    yCom = np.sum(pos[1] * mass) / totMass

    # Apply the density perturbation
    for i in range(ngas):
        # Find relative positions
        x = xCom - pos[0,i]
        y = yCom - pos[1,i]

        # Work out the angle 
        phi = np.arctan2(y, x)

        # Work out what the mass should be here
        mass[i] = mass[i] * (1 + 0.5 * np.cos(2*phi))

    return pos, mass

def densityGradient(pos, mass, lowerDensity=0.66, upperDensity=1.33):
    distance = pos[0] + np.max(pos[0])
    return lowerDensity * mass + (upperDensity-lowerDensity) * mass * (distance/np.max(distance))

def bonnorEbert(ngas, pos, mass, temp, mu, beMass):
    # Calculate the sound speed
    cs = np.sqrt(kB * temp / (mu * 1.66e-24))
    beMass = beMass * 1.991e33
    
    # Calculate characteristic quantities 
    rBonnorEbert = 3.09e18 * 0.0043016 * (beMass/1.991e33) * 6.5 / ((cs/1e5)**2 * 15.85)
    print("Bonnor Ebert Radius: {:.2e}".format(rBonnorEbert))
     
    outsideRho = 1.66e-22
    centralDensity = 14.305 * outsideRho
    rCharacteristic = cs / (np.sqrt(4 * np.pi * G * centralDensity))

    #centralDensity = (6.5**2 / np.sqrt(4 * np.pi)) * (cs**2 / G) * (1/rBonnorEbert**2)
    print("Central Density: {:.2e}".format(centralDensity))
    #rCharacteristic = cs / np.sqrt(4 * np.pi * G * centralDensity)
    print("Characteristic Radius: {:.2e}".format(rCharacteristic))
    
    # Create bins of radius and corresponding density
    radiusBins = 10**np.linspace(0, np.log10(rBonnorEbert), 1000)
    densityBins = centralDensity * rCharacteristic**2 / (rCharacteristic**2 + radiusBins**2)

    # Find the centre of mass
    totMass = np.sum(mass)
    xC = np.sum(pos[0] * mass) / totMass
    yC = np.sum(pos[1] * mass) / totMass
    zC = np.sum(pos[2] * mass) / totMass
    
    # Find each particle's distance to the COM
    rCentre = np.sqrt((pos[0])**2 + (pos[1])**2 + (pos[2])**2)
    
    # Scale everything to the correct radius
    pos[0] = rBonnorEbert * pos[0] / np.max(rCentre) 
    pos[1] = rBonnorEbert * pos[1] / np.max(rCentre)
    pos[2] = rBonnorEbert * pos[2] / np.max(rCentre)
    rCentre = rBonnorEbert * rCentre / np.max(rCentre) 
    
    # Scale the mass of each particle
    for i in range(len(radiusBins)-1): 
        inRadius = np.where((rCentre > radiusBins[i]) & (rCentre < radiusBins[i+1]))
            
        # Work out how much mass should be in each shell
        shellMass = (4 * np.pi / 3) * (radiusBins[i+1]**3 - radiusBins[i]**3) * densityBins[i] 
        
        # Find the total mass of particles in this shell and scale
        cellMass = shellMass / len(inRadius[0])
        
        # Assign value
        mass[inRadius] = cellMass
        
    # Scale masses
    totMass = np.sum(mass)
    mass = mass * (beMass / totMass)
        
    # Work out cloud density
    volume = ((4 * np.pi / 3) * rBonnorEbert**3)
    cloudDensity = beMass / volume
    densityFrac = np.min(densityBins)/cloudDensity

    return mass, pos, densityFrac/15, volume/(3.09e18**3)

# Create a centrally condensed density profile
def centrallyCondensedSphere(ngas, pos, pMass, mass):
    # Calculate the radius of the sphere
    centralDensity = 1e-20
    flatFraction = 0.1
    rSphere = (mass * 1.991e33 / (4 * np.pi * centralDensity * flatFraction**3 * (-1 * np.arctan(1/flatFraction) + (1/flatFraction))))**(1/3)
    print("Creating Sphere of Radius {:.2e} m".format(rSphere))
    
    # Calculate the centre of mass 
    xcom = np.sum(pos[0] * pMass) / np.sum(pMass)
    ycom = np.sum(pos[1] * pMass) / np.sum(pMass)
    zcom = np.sum(pos[2] * pMass) / np.sum(pMass)
    
    # Calculate radial distance to the CoM
    rCentre = np.sqrt((pos[0] - xcom)**2 + (pos[1] - ycom)**2 + (pos[2] - zcom)**2)
    rSphereCurrent = np.max(rCentre)
    
    # Scale positions to the correct radius
    pos[0] = pos[0] * (rSphere / rSphereCurrent)
    pos[1] = pos[1] * (rSphere / rSphereCurrent)
    pos[2] = pos[2] * (rSphere / rSphereCurrent)
    rCentre = rCentre * (rSphere / rSphereCurrent)
    
    # Scale cell masses by the density function
    cloudVol = 4 * np.pi * rSphere**3 / 3
    cellVol = cloudVol / ngas
    cellDensity = centralDensity * (1 + (rCentre / (flatFraction * rSphere))**2)**(-1)
    pMass = cellVol * cellDensity

    # Work out density at the edge of the sphere
    cloudRho = mass * 1.991e33 / cloudVol
    densityFraction = 0.01 * np.min(cellDensity) / cloudRho
        
    return pos, pMass, cloudVol/(3.09e18)**3, densityFraction
